package me.ntfwc.blockLoggingPlugin;

import java.util.LinkedList;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
public abstract class BlockLoggingDatabase
{
	private static String RELATIVE_DIRECTORY = "plugins/BlockLogging/blockLogDatabase";
	private static String TABLE_NAME = "BLOCK_LOG_TABLE";
	private static int PREFERRED_NUM_RECORDS_PER_BLOCK = 10;
	private static final int ROW_REMOVAL_MARGIN = 5;
	private static int ROW_REMOVAL_CEILING = PREFERRED_NUM_RECORDS_PER_BLOCK + ROW_REMOVAL_MARGIN;
	private static PreparedStatement blockChangeInsertionStatement;
	private static PreparedStatement blockChangeQueryStatement;
	//private static PreparedStatement blockChangeQueryByUsernameStatement;
	private static PreparedStatement blockChangeQueryByUsernameAndBoundariesStatement;
	private static PreparedStatement blockChangeDeletionStatement;
	//private static PreparedStatement blockChangeDeletionByUsernameStatement;
	//private static PreparedStatement blockChangeDeletionByUsernameAndBoundariesStatement;
	private static void createPreparedStatements() throws SQLException
	{
		blockChangeInsertionStatement = DerbySQLDatabase.createPreparedStatement("INSERT INTO " + TABLE_NAME + " (x, y, z, username, worldName, block_id1, block_data1, block_id2, block_data2, time) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
		blockChangeQueryStatement = DerbySQLDatabase.createPreparedStatement("SELECT id, username, block_id1, block_data1, block_id2, block_data2, time FROM " + TABLE_NAME + " WHERE worldName = ? AND x = ? AND y = ? AND z = ? ORDER BY time");
		//blockChangeQueryByUsernameStatement = DerbySQLDatabase.createPreparedStatement("SELECT x, y, z, worldName, block_id1, block_data1, block_id2, block_data2, time FROM " + TABLE_NAME + " WHERE username = ? ORDER BY time");
		blockChangeQueryByUsernameAndBoundariesStatement = DerbySQLDatabase.createPreparedStatement("SELECT x, y, z, block_id1, block_data1, block_id2, block_data2, time FROM " + TABLE_NAME + " WHERE username = ? AND worldName = ? AND x >= ? AND y >= ? AND z >= ? AND x <= ? AND y <= ? AND z <= ? ORDER BY time");
		blockChangeDeletionStatement = DerbySQLDatabase.createPreparedStatement("DELETE FROM " + TABLE_NAME + " WHERE id = ?");
		//blockChangeDeletionByUsernameStatement = DerbySQLDatabase.createPreparedStatement("DELETE FROM " + TABLE_NAME + " WHERE username = ?");
		//blockChangeDeletionByUsernameAndBoundariesStatement = DerbySQLDatabase.createPreparedStatement("DELETE FROM " + TABLE_NAME + " WHERE username = ? AND worldName = ? AND x >= ? AND y >= ? AND z >= ? AND x <= ? AND y <= ? AND z <= ?");
	}
	
	public static void start(int historySize) throws SQLException
	{
		PREFERRED_NUM_RECORDS_PER_BLOCK = historySize;
		ROW_REMOVAL_CEILING = PREFERRED_NUM_RECORDS_PER_BLOCK + ROW_REMOVAL_MARGIN;
		
		if (!DerbySQLDatabase.isActive())
		{
			DerbySQLDatabase.startDatabase(RELATIVE_DIRECTORY);
		}
		LinkedList<String> tableNames = DerbySQLDatabase.getListOfTableNames();
		if (!tableNames.contains(TABLE_NAME))
		{
			DerbySQLDatabase.createTableWithGeneratedKey(TABLE_NAME, "x INT, y INT, z INT, username VARCHAR(16), worldName VARCHAR(40), block_id1 INT, block_data1 SMALLINT, block_id2 INT, block_data2 SMALLINT, time BIGINT");
		}
		createPreparedStatements();
	}
	private static void storeBlockChange(LoggableBlockChange loggableBlockChange) throws SQLException
	{
		blockChangeInsertionStatement.setInt(1, loggableBlockChange.x);
		blockChangeInsertionStatement.setInt(2, loggableBlockChange.y);
		blockChangeInsertionStatement.setInt(3, loggableBlockChange.z);
		blockChangeInsertionStatement.setString(4, loggableBlockChange.username);
		blockChangeInsertionStatement.setString(5, loggableBlockChange.worldName);
		blockChangeInsertionStatement.setInt(6, loggableBlockChange.block_id1);
		blockChangeInsertionStatement.setByte(7, loggableBlockChange.block_data1);
		blockChangeInsertionStatement.setInt(8, loggableBlockChange.block_id2);
		blockChangeInsertionStatement.setByte(9, loggableBlockChange.block_data2);
		blockChangeInsertionStatement.setLong(10, loggableBlockChange.time);
		blockChangeInsertionStatement.executeUpdate();
	}
	private static void sendUsernameAndBoundaryParameters(PreparedStatement statement, String username, String worldName, int minX, int minY, int minZ, int maxX, int maxY, int maxZ) throws SQLException
	{
		statement.setString(1, username);
		statement.setString(2, worldName);
		statement.setInt(3, minX);
		statement.setInt(4, minY);
		statement.setInt(5, minZ);
		statement.setInt(6, maxX);
		statement.setInt(7, maxY);
		statement.setInt(8, maxZ);
	}
	private static ResultSet getBlockChangeResultSet(String worldName, int x, int y, int z) throws SQLException
	{
		blockChangeQueryStatement.setString(1, worldName);
		blockChangeQueryStatement.setInt(2, x);
		blockChangeQueryStatement.setInt(3, y);
		blockChangeQueryStatement.setInt(4, z);
		return blockChangeQueryStatement.executeQuery();
	}
	private static void deleteBlockChange(long id) throws SQLException
	{
		blockChangeDeletionStatement.setLong(1, id);
		blockChangeDeletionStatement.executeUpdate();
	}
	/*
	public static void deleteBlockChangesByUsername(String username) throws SQLException
	{
		blockChangeDeletionByUsernameStatement.setString(1, username);
		blockChangeDeletionByUsernameStatement.executeUpdate();
	}
	public static void deleteBlockChangesByUsernameInBounds(String username, String worldName, int minX, int minY, int minZ, int maxX, int maxY, int maxZ) throws SQLException
	{
		sendUsernameAndBoundaryParameters(blockChangeDeletionByUsernameAndBoundariesStatement, username, worldName, minX, minY, minZ, maxX, maxY, maxZ);
		blockChangeDeletionByUsernameAndBoundariesStatement.executeUpdate();
	}
	*/
	private static int getNumOfRecordedBlockChanges(String worldName, int x, int y, int z) throws SQLException
	{
		ResultSet result = getBlockChangeResultSet(worldName, x, y, z);
		int count = 0;
		while (result.next())
		{
			count++;
		}
		return count;
	}
	private static void deleteNextItem(ResultSet result) throws SQLException
	{
		result.next();
		int id = result.getInt(1);
		deleteBlockChange(id);
	}
	private static void removeExtraRows(String worldName, int x, int y, int z) throws SQLException
	{
		int count = getNumOfRecordedBlockChanges(worldName, x, y, z);
		if (count >= ROW_REMOVAL_CEILING)
		{
			ResultSet result = getBlockChangeResultSet(worldName, x, y, z);
			deleteNextItem(result);
			count--;
			while (count >= PREFERRED_NUM_RECORDS_PER_BLOCK)
			{
				deleteNextItem(result);
				count--;
			}
		}
	}
	public static void store(LoggableBlockChange loggableBlockChange) throws SQLException
	{
		removeExtraRows(loggableBlockChange.worldName, loggableBlockChange.x, loggableBlockChange.y, loggableBlockChange.z);
		storeBlockChange(loggableBlockChange);
	}
	public static LinkedList<LoggableBlockChange> getBlockChangesForLocation(String worldName, int x, int y, int z) throws SQLException
	{
		ResultSet result = getBlockChangeResultSet(worldName, x, y, z);
		LinkedList<LoggableBlockChange> blockChanges = new LinkedList<LoggableBlockChange>();
		while (result.next())
		{
			String username = result.getString(2);
			int block_id1 = result.getInt(3);
			byte block_data1 = result.getByte(4);
			int block_id2 = result.getInt(5);
			byte block_data2 = result.getByte(6);
			long time = result.getLong(7);
			blockChanges.add(new LoggableBlockChange(x, y, z, username, worldName, block_id1, block_data1, block_id2, block_data2, time));
		}
		return blockChanges;
	}
	/*private static LinkedList<LoggableBlockChange> getListOfBlockChangesGivenUsername(ResultSet result, String username) throws SQLException
	{
		LinkedList<LoggableBlockChange> blockChanges = new LinkedList<LoggableBlockChange>();
		while (result.next())
		{
			int x = result.getInt(1);
			int y = result.getInt(2);
			int z = result.getInt(3);
			String worldName = result.getString(4);
			int block_id1 = result.getInt(5);
			byte block_data1 = result.getByte(6);
			int block_id2 = result.getInt(7);
			byte block_data2 = result.getByte(8);
			long time = result.getLong(9);
			blockChanges.add(new LoggableBlockChange(x, y, z, username, worldName, block_id1, block_data1, block_id2, block_data2, time));
		}
		return blockChanges;
	}
	*/
	/*public static LinkedList<LoggableBlockChange> getBlockChangesForUser(String username) throws SQLException
	{
		blockChangeQueryByUsernameStatement.setString(1, username);
		ResultSet result = blockChangeQueryByUsernameStatement.executeQuery();
		return getListOfBlockChangesGivenUsername(result, username);
	}
	*/
	private static LinkedList<LoggableBlockChange> getListOfBlockChangesGivenUsernameAndWorldName(ResultSet result, String username, String worldName) throws SQLException
	{
		LinkedList<LoggableBlockChange> blockChanges = new LinkedList<LoggableBlockChange>();
		while (result.next())
		{
			int x = result.getInt(1);
			int y = result.getInt(2);
			int z = result.getInt(3);
			int block_id1 = result.getInt(4);
			byte block_data1 = result.getByte(5);
			int block_id2 = result.getInt(6);
			byte block_data2 = result.getByte(7);
			long time = result.getLong(8);
			blockChanges.add(new LoggableBlockChange(x, y, z, username, worldName, block_id1, block_data1, block_id2, block_data2, time));
		}
		return blockChanges;
	}
	public static LinkedList<LoggableBlockChange> getBlockChangesForUserInBounds(String username, String worldName, int minX, int minY, int minZ, int maxX, int maxY, int maxZ) throws SQLException
	{
		sendUsernameAndBoundaryParameters(blockChangeQueryByUsernameAndBoundariesStatement, username, worldName, minX, minY, minZ, maxX, maxY, maxZ);
		ResultSet result = blockChangeQueryByUsernameAndBoundariesStatement.executeQuery();
		return getListOfBlockChangesGivenUsernameAndWorldName(result, username, worldName);
	}
	public static void stop() throws SQLException
	{
		if (DerbySQLDatabase.isActive())
		{
			DerbySQLDatabase.shutdownDatabase();
		}
	}
}