package me.ntfwc.blockLoggingPlugin;


import java.sql.SQLException;

import org.bukkit.Server;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.scheduler.BukkitScheduler;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;

public class BlockLoggingPlugin extends JavaPlugin{
	private static final long INITIAL_DELAY = 100;
	private static final long BLOCK_TASK_DELAY = 20;
	//private static final long BLOCK_LOGGING_THREAD_DELAY = 10;
	private static final long PLAYER_RESULT_FETCHER_DELAY = 20;
	
	private Thread blockLoggingRunnableThread;
	
	
	private void startBlockLoggingRunnable(BlockLoggingRunnable blockLoggingRunnable)
	{
		this.blockLoggingRunnableThread = new Thread(blockLoggingRunnable);
		blockLoggingRunnableThread.start();
	}
	
	
	
	private int getHistorySize()
	{
		return ConfigControl.getInt(this, "history_size");
	}
	
    //private final TestPluginPlayerListener playerListener = new TestPluginPlayerListener(this);
    public void onEnable(){ 
    	//start database
    	try
    	{
    		BlockLoggingDatabase.start(getHistorySize());
    	}
    	catch (SQLException e)
    	{
    		e.printStackTrace();
    		return;
    	}
    	
    	//get the server
    	Server server = this.getServer();
    	BlockLoggingRunnable blockLoggingRunnable = new BlockLoggingRunnable();
    	MessageSender messageSender = new MessageSender();
    	CoreAPI.init(server, blockLoggingRunnable, messageSender);
    	
    	//set up scheduled threads
    	BukkitScheduler scheduler = server.getScheduler();
    	
    	scheduler.scheduleSyncRepeatingTask(this, new IterativeBlockOperationRunner(), INITIAL_DELAY, BLOCK_TASK_DELAY);
    	scheduler.scheduleSyncRepeatingTask(this, messageSender, INITIAL_DELAY, PLAYER_RESULT_FETCHER_DELAY);
    	
    	//set up listeners
    	PluginManager pm = this.getServer().getPluginManager();
    	pm.registerEvents(new BlockListener(), this);
    	pm.registerEvents(new PlayerListener(), this);
    	
    	startBlockLoggingRunnable(blockLoggingRunnable);
    	//done
    	
    	CoreAPI.log("NtfLog plugin enabled.");
    }
    
    private void stopBlockLoggingRunnable()
	{
    	if (this.blockLoggingRunnableThread != null)
    	{
    		blockLoggingRunnableThread.interrupt();
    	}
	}
     
    public void onDisable(){ 
    	stopBlockLoggingRunnable();
    	
    	try
    	{
    		BlockLoggingDatabase.stop();
    	}
    	catch (SQLException e)
    	{
    		e.printStackTrace();
    	}
    	
    	CoreAPI.log("NtfLog plugin disabled.");
    }
    public boolean onCommand(CommandSender sender, Command cmd, String commandLabel, String[] args)
	{
		if (sender instanceof Player) {
			Player player = (Player) sender;
			return PlayerCommandProcessing.processPlayerCommand(player, cmd, commandLabel, args);
		}
		else if (sender instanceof ConsoleCommandSender)
		{
			ConsoleCommandSender console = (ConsoleCommandSender) sender;
			return ConsoleCommandProcessing.processConsoleWindowCommand(console, cmd, commandLabel, args);
		}
		
		
		return false;
	}
}
