package me.ntfwc.blockLoggingPlugin;

import java.util.logging.Logger;

import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Server;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;

public abstract class CoreAPI {
	private static Logger log = Logger.getLogger("Minecraft");
	private static Server server;
	public static BlockLoggingRunnable blockLoggingRunnable;
	public static MessageSender messageSender;
	public static PlayerDataDictionary playerDataDict = new PlayerDataDictionary();
	
	
	
	//server wide
	public static void init(Server server, BlockLoggingRunnable blockLoggingRunnable, MessageSender messageSender)
	{
		CoreAPI.server = server;
		CoreAPI.blockLoggingRunnable = blockLoggingRunnable;
		CoreAPI.messageSender = messageSender;
	}
	
	public static void log(String s)
	{
		log.info(s);
	}
	
	//console
	public static void sendMessage(ConsoleCommandSender console, String message)
	{
		console.sendMessage(message);
	}
	
	//players
	public static void sendMessage(Player player, String message)
	{
		player.sendMessage(message);
	}
	
	public static void asyncSendMessage(Player player, String message)
	{
		messageSender.addTask(new PlayerOutputTask(player, message));
	}
	
	/*public static Location getLocation(Player player)
	{
		return player.getLocation();
	}
	
	public static void teleportPlayerToLocation(Player player, Location location)
	{
		player.teleport(location);
	}*/
	
	/*public static World getPlayerWorld(Player player)
	{
		return player.getWorld();
	}*/
	
	public static PlayerDataContainer getPlayerDataContainer(Player player)
	{
		return playerDataDict.getPlayerDataContainer(player);
	}
	
	public static String getPlayerName(Player player)
	{
		return player.getName();
	}
	
	//blocks
	private static boolean setBlock(Block block, Material type)
	{
		Material currentType = getBlockType(block);
		if (currentType != type)
		{
			block.setType(type);
			return true;
		}
		return false;
	}
	public static void setBlock(Block block, Material type, boolean isSpecialType, byte data)
	{
		boolean dataReset = setBlock(block, type);
		if (isSpecialType)
		{
			if (!(data == (byte) 0x00 && dataReset))
			{
				setBlockData(block, data);
			}
		}
	}
	
	public static void setBlock(Block block, ExtendedBlockMaterial extendedType)
	{
		setBlock(block, extendedType.getMaterial(), extendedType.isSpecialType(), extendedType.getData());
	}
	
	private static void setBlockData(Block block, byte data)
	{
		byte currentData = getBlockData(block);
		if (currentData != data)
		{
			block.setData(data);
		}
	}
	
	/*public static void setBlockAtLocation(Location location, ExtendedBlockMaterial type)
	{
		if (isInWorld(location))
		{
			setBlock(getBlockAtLocation(location), type);
		}
		
	}*/
	
	/*public static void setBlockAtLocation(Location location, Material type, boolean isSpecialType, byte data)
	{
		if (isInWorld(location))
		{
			setBlock(getBlockAtLocation(location), type, isSpecialType, data);
		}
		
	}*/
	
	public static void setBlockAtLocation(World world, int x, int y, int z, ExtendedBlockMaterial type)
	{
		if (isInWorld(world, y))
		{
			setBlock(getBlockAtLocation(world, x, y, z), type);
		}
	}
	
	public static void asyncSetBlockAtLocation(World world, int x, int y, int z, ExtendedBlockMaterial type)
	{
		BlockTaskQueuing.addTask(new BlockTask(world,x,y,z,type));
	}
	
	/*public static void setBlockAtLocation(World world, int x, int y, int z, Material type, boolean isSpecialType, byte data)
	{
		if (isInWorld(world, y))
		{
			setBlock(getBlockAtLocation(world, x, y, z), type, isSpecialType, data);
		}
	}*/
	
	
	public static Material getBlockType(Block block)
	{
		return block.getType();
	}
	
	public static byte getBlockData(Block block)
	{
		return block.getData();
	}
	
	public static Location getBlockLocation(Block block)
	{
		return block.getLocation();
	}
	
	/*public static World getBlockWorld(Block block)
	{
		return block.getWorld();
	}*/
	
	/*public static Block getBlockAtLocation(Location location)
	{
		return location.getBlock();
	}*/
	
	public static Block getBlockAtLocation(World world, int x, int y, int z)
	{
		return world.getBlockAt(x, y, z);
	}
	//locations
	public static int getX(Location location)
	{
		return location.getBlockX();
	}
	public static int getY(Location location)
	{
		return location.getBlockY();
	}
	public static int getZ(Location location)
	{
		return location.getBlockZ();
	}
	public static World getWorld(Location location)
	{
		return location.getWorld();
	}
	
	public static String getWorldName(Location location)
	{
		return location.getWorld().getName();
	}
	
	public static Location tryToGetBlockLocation(Block block)
	{
		if (block == null)
		{
			return null;
		}
		
		return getBlockLocation(block);
	}
	
	public static Location getLastBlockEditLocation(Player player)
	{
		PlayerDataContainer dataContainer = CoreAPI.getPlayerDataContainer(player);
		Block lastBlockChanged = dataContainer.getBlockChanged(0);
		
		return tryToGetBlockLocation(lastBlockChanged);
	}
	
	public static Tuple2<Location,Location> getLastTwoBlockEditLocations(Player player)
	{
		PlayerDataContainer dataContainer = CoreAPI.getPlayerDataContainer(player);
		Block lastBlockChanged = dataContainer.getBlockChanged(0);
		Block secondToLastBlockChanged = dataContainer.getBlockChanged(1);
		
		Location location1 = tryToGetBlockLocation(lastBlockChanged);
		Location location2 = tryToGetBlockLocation(secondToLastBlockChanged);
		
		return new Tuple2<Location,Location>(location1, location2);
	}
	
	//materials
	public static int getId(Material material)
	{
		return material.getId();
	}
	
	public static Material getMaterial(int id)
	{
		return Material.getMaterial(id);
	}
	
	//extended material
	public static ExtendedBlockMaterial getExtendedBlockMaterial(int id, byte data)
	{
		return new ExtendedBlockMaterial(getMaterial(id), data);
	}
	
	//Worlds
	public static int getMaxHeight(World world)
	{
		return world.getMaxHeight();
	}
	
	public static boolean isInWorld(World world, int y)
	{
		return 0 <= y && y < getMaxHeight(world);
	}
	
	/*public static boolean isInWorld(Location location)
	{
		return isInWorld(getWorld(location), getY(location));
	}*/
	
	public static World getWorld(String worldName)
	{
		return server.getWorld(worldName);
	}
}
