package me.ntfwc.blockLoggingPlugin;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.sql.DatabaseMetaData;
import java.util.LinkedList;
public abstract class DerbySQLDatabase
{
	private static boolean isActive = false;
	private static Connection connection;
	private static Statement staticStatment;
	private static Connection getConnectionToDataBase(String databasePath) throws SQLException
	{
		try
		{
			String driver = "org.apache.derby.jdbc.EmbeddedDriver";
			Class.forName(driver);
		}
		catch (Exception e)
		{
			throw new SQLException("Could not find driver");
		}
		return DriverManager.getConnection("jdbc:derby:" + databasePath + ";create=true");
	}
	public static void startDatabase(String databasePath) throws SQLException
	{
		connection = getConnectionToDataBase(databasePath);
		staticStatment = connection.createStatement();
		isActive = true;
	}
	public static void shutdownDatabase() throws SQLException
	{
		staticStatment.close();
		staticStatment = null;
		connection.commit();
		connection.close();
		connection = null;
		try
		{
			DriverManager.getConnection("jdbc:derby:;shutdown=true");
		}
		catch (SQLException e) {}
		isActive = false;
	}
	public static boolean isActive()
	{
		return isActive;
	}
	public static String[] getResultSetColumnNames(ResultSet resultSet) throws SQLException
	{
		ResultSetMetaData metaData = resultSet.getMetaData();
		int numColumns = metaData.getColumnCount();
		String[] columnNames = new String[numColumns];
		for (int i = 1; i < numColumns+1; i++)
		{
			columnNames[i-1] = metaData.getColumnName(i);
		}
		return columnNames;
	}
	public static int[] getResultSetColumnTypes(ResultSet resultSet) throws SQLException
	{
		ResultSetMetaData metaData = resultSet.getMetaData();
		int numColumns = metaData.getColumnCount();
		int[] columnTypes = new int[numColumns];
		for (int i = 1; i < numColumns; i++)
		{
			columnTypes[i] = metaData.getColumnType(i);
		}
		return columnTypes;
	}
	public static LinkedList<String> getListOfTableNames() throws SQLException
	{
		String[] types = {"TABLE"};
		DatabaseMetaData metaData = connection.getMetaData();
		ResultSet resultSet = metaData.getTables(null,null,"%",types);
		LinkedList<String> nameList = new LinkedList<String>();
		while (resultSet.next())
		{
			String tableName = resultSet.getString("TABLE_NAME");
			nameList.add(tableName);
		}
		return nameList;
	}
	public static void executeSqlStatement(String statement) throws SQLException
	{
		staticStatment.execute(statement);
	}
	public static void executeSqlUpdate(String updateStatement) throws SQLException
	{
		staticStatment.executeUpdate(updateStatement);
	}
	public static ResultSet getSqlQueryResult(String queryStatment) throws SQLException
	{
		return staticStatment.executeQuery(queryStatment);
	}
	public static PreparedStatement createPreparedStatement(String sqlStatment) throws SQLException
	{
		return connection.prepareStatement(sqlStatment);
	}
	public static void createTableWithGeneratedKey(String name, String columns) throws SQLException
	{
		executeSqlStatement("CREATE TABLE " + name + " (id BIGINT PRIMARY KEY GENERATED ALWAYS AS IDENTITY NOT NULL, " + columns + ")");
	}
	public static void deleteTable(String tableName) throws SQLException
	{
		executeSqlStatement("DROP TABLE " + tableName);
	}
	public static void deleteTableIfItExists(String tableName) throws SQLException
	{
		executeSqlStatement("DROP TABLE " + tableName + " IF EXISTS");
	}
	public static ResultSet getAllEntriesInTable(String tableName) throws SQLException
	{
		return getSqlQueryResult("SELECT * FROM " + tableName);
	}
}