package me.ntfwc.blockLoggingPlugin;

import java.sql.SQLException;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

import org.bukkit.Location;
import org.bukkit.World;

public class LoggingTask_RollbackRegion implements LoggingTask{
	String username;
	Location location1;
	Location location2;
	
	public LoggingTask_RollbackRegion(String username, Location location1, Location location2)
	{
		this.username = username.toLowerCase();
		this.location1 = location1;
		this.location2 = location2;
	}
	
	private static void update(LoggableBlockChange blockChange, LinkedList<LoggableBlockChange> blockChanges, Set<Point3d> recordedPositions)
	{
		Point3d point = new Point3d(blockChange.x, blockChange.y, blockChange.z);
		if (!recordedPositions.contains(point))
		{
			blockChanges.add(blockChange);
			recordedPositions.add(point);
		}
	}
	
	private static LinkedList<LoggableBlockChange> getOldestChanges(LinkedList<LoggableBlockChange> blockChangeList)
	{
		Collections.sort(blockChangeList);
		LinkedList<LoggableBlockChange> oldestBlockChanges = new LinkedList<LoggableBlockChange>();
		Set<Point3d> recordedPositions = new HashSet<Point3d>();
		for (LoggableBlockChange blockChange : blockChangeList)
		{
			update(blockChange, oldestBlockChanges, recordedPositions);
		}
		return oldestBlockChanges;
	}
	
	public void run() throws SQLException
	{
		String worldName = CoreAPI.getWorldName(location1);
		int location1X = CoreAPI.getX(location1);
		int location1Y = CoreAPI.getY(location1);
		int location1Z = CoreAPI.getZ(location1);
		int location2X = CoreAPI.getX(location2);
		int location2Y = CoreAPI.getY(location2);
		int location2Z = CoreAPI.getZ(location2);
		
		int minX;
		int maxX;
		if (location1X < location2X)
		{
			minX = location1X;
			maxX = location2X;
		}
		else
		{
			minX = location2X;
			maxX = location1X;
		}
		int minY;
		int maxY;
		if (location1Y < location2Y)
		{
			minY = location1Y;
			maxY = location2Y;
		}
		else
		{
			minY = location2Y;
			maxY = location1Y;
		}
		int minZ;
		int maxZ;
		if (location1Z < location2Z)
		{
			minZ = location1Z;
			maxZ = location2Z;
		}
		else
		{
			minZ = location2Z;
			maxZ = location1Z;
		}
		
		LinkedList<LoggableBlockChange> blockChangeList = BlockLoggingDatabase.getBlockChangesForUserInBounds(username, worldName, minX, minY, minZ, maxX, maxY, maxZ);
		LinkedList<LoggableBlockChange> oldestBlockChangeList = getOldestChanges(blockChangeList);
		World world = CoreAPI.getWorld(worldName);
		
		for (LoggableBlockChange blockChange : oldestBlockChangeList)
		{
			blockChange.rollback(world);
		}
	}

}
